<?php

/**
 * @file
 * BitPay webhook handler.
 */

/**
 * Page callback for BitPay notification URL.
 */
function bitpay_callback() {
  if ($client = bitpay_initialize()) {
    if ($request = json_decode(file_get_contents('php://input'))) {
      try {
        $invoice = $client->getInvoice($request->id);
      }
      catch (Exception $e) {
        watchdog('bitpay', 'Error processing IPN: %message', array('%message' => $e->getMessage()), WATCHDOG_ERROR);
      }
      if (!empty($invoice)) {
        bitpay_record_invoice($invoice);
        $response = json_decode($client->getResponse()->getBody(), TRUE);
        module_invoke_all('bitpay_invoice_update', $response['data']);
      }
    }
    return;
  }
  // Return if client failed to initialize.
  if ($client === FALSE) {
    return;
  }
  $invoice = bpVerifyNotification();
  if (variable_get('bitpay_debug', FALSE)) {
    watchdog('debug', 'Receiving BitPay invoice status update: %invoice.', array('%invoice' => print_r($invoice, TRUE)), WATCHDOG_DEBUG);
  }
  if (!is_array($invoice)) {
    watchdog('bitpay', 'Error while processing BitPay invoice update notification: %message.', array('%message' => strval($invoice)), WATCHDOG_ERROR);
    return MENU_ACCESS_DENIED;
  }
  // Update the record for this invoice.
  $record['btc_due'] = $invoice['btcDue'];
  $record['btc_paid'] = $invoice['btcPaid'];
  $record['btc_price'] = $invoice['btcPrice'];
  $record['currency'] = $invoice['currency'];
  $record['exception_status'] = $invoice['exceptionStatus'];
  $record['price'] = $invoice['price'];
  $record['rate'] = $invoice['rate'];
  $record['status'] = $invoice['status'];
  $record['pos_data'] = $invoice['posData'];
  if (isset($invoice['buyerFields']['buyerEmail'])) {
    $record['buyer_email'] = $invoice['buyerFields']['buyerEmail'];
  }
  bitpay_write_record($record, $invoice['id']);
  module_invoke_all('bitpay_invoice_update', $invoice);
}
